--------------
-- Domino Deus --
--------------
-- a Domino game by Ghuntar WALDMEISTER
-- GAME_class.lua


Game =  {
        playerTurn = 0,
        playerWin = false,
        ended = false,
        }

function Game:new()
    newobject = {}
    setmetatable(newobject, self)
    self.__index = self
    -- The boneyard is where players draw theire bones/tiles.
    newobject.boneYard = BoneYard:new()
    -- Generation of all the 28 bones and there insertion in the boneyard.
    for x=0 , 6 do
        for y=x, 6 do
            table.insert(newobject.boneYard,Bone:new({x,y}))
        end
    end

    -- Player's Hands
    newobject.hands = {}
    for i=1 , config.numberOfPlayer do
        newobject.hands[i] = BoneSet:new()
    end

     -- Distribution of the bones
    newobject.playerTurn = 0
    while (#newobject.boneYard>0) do
        newobject.playerTurn = (newobject.playerTurn)%config.numberOfPlayer +1
        -- Each player receives 7 bones
        if #newobject.hands[newobject.playerTurn]>6 then break end
        local key = math.random(1,#newobject.boneYard)
        table.insert(newobject.hands[newobject.playerTurn],newobject.boneYard[key])
        table.remove(newobject.boneYard,key)
    end

    -- firstBone is the first bone that will be played (it should be double 6 in most case)
    -- if the boneyard is empty, one player must have the double 6
    if #newobject.boneYard == 0
    then newobject.firstBone = Bone:new({6,6})
    -- if not, the doubles are tried one by one from the highest to the lowest,
    -- As soon as one of the double is declared absent from the boneyard, that implies that one player has it
    else
        for i = 6, 0, -1 do
            if not Bone:new({i,i}):isPresentInTable(newobject.boneYard)
            then newobject.firstBone = Bone:new({i,i}) break
            end
        end
    end
    -- if all doubles are in the boneyard (quite unlikely), the search continue in that order : {6,5},{6,4},...,{0,2},{0,1}
    if not newobject.firstBone 
    then
        for x = 6, 0, -1 do
            for y = 6, 0, -1 do
                if not Bone:new({x,y}):isPresentInTable(newobject.boneYard)
                then newobject.firstBone = Bone:new({x,y}) break
                end
            end
            if newobject.firstBone then break end
        end
    end

    -- The field is where the bones will be played (empty at the begining).
    newobject.field = Field:new{}
    newobject.field:setCoord(400,300)
    -- Looking in each player's hand the firstBone to know who begins
    for playerNumber,playerHand in ipairs(newobject.hands) do
        for k,v in ipairs(playerHand) do
            if v:fullMatch(newobject.firstBone)
            then
                table.insert(newobject.field,v)
                table.remove(playerHand,k)
                playerHand.hasPlayed = true
                -- print("Bone "..v:tostring().." from Player"..playerNumber.." to field")
                newobject.playerTurn = playerNumber
            end
        end
    end
    return newobject
end

function Game:update(dt)
    if not self.ended
    then
        if self.hands[self.playerTurn].hasPlayed
        then
            self.playerTurn = (self.playerTurn) % config.numberOfPlayer +1
        end
         -- check if current player is player1 (Human)
        if self.playerTurn == 0 --set to 0 to play in full CPU mode
        then
            -- code to manage the human player
            self.hands[self.playerTurn].hasPlayed = false
            self.hands[self.playerTurn].hasPassed = false
            self.hands[self.playerTurn].updated = false

            
            

            self.hands[self.playerTurn].hasPlayed = false
            self.hands[self.playerTurn].hasPassed = true
        else
            --code to manage CPU players
            self.hands[self.playerTurn].hasPlayed = false
            self.hands[self.playerTurn].hasPassed = false
            self.hands[self.playerTurn].updated = false
            while not self.hands[self.playerTurn].hasPlayed do
                -- browse the current player's hand
                for index, bone in ipairs(self.hands[self.playerTurn]) do
                    -- check if the bone selected match with the "first" value of the field (first value of the first bone)
                    if bone:matchValue(self.field[1][1])
                    -- if false
                    then
                        table.insert(self.field,1,bone)
                        if self.field[1][2] ~= self.field[2][1] then self.field[1]:switch() end
                        self.field.updated = true
                        table.remove(self.hands[self.playerTurn],index)
                        self.hands[self.playerTurn].updated = true
                        self.hands[self.playerTurn].hasPlayed = true
                        break
                    elseif bone:matchValue(self.field[#self.field][2])
                    then
                        table.insert(self.field,bone)
                        if self.field[#self.field][1] ~= self.field[#self.field-1][2] then self.field[#self.field]:switch() end
                        self.field.updated = true
                        table.remove(self.hands[self.playerTurn],index)
                        self.hands[self.playerTurn].updated = true
                        self.hands[self.playerTurn].hasPlayed = true
                        break
                    end
                end
                if not self.hands[self.playerTurn].hasPlayed
                then
                    if #self.boneYard>0
                    then
                        key = math.random(1,#self.boneYard)
                        table.insert(self.hands[self.playerTurn],self.boneYard[key])
                        table.remove(self.boneYard,key)
                        self.hands[self.playerTurn].hasPlayed = true
                    else
                        self.hands[self.playerTurn].hasPlayed = true
                        self.hands[self.playerTurn].hasPassed = true
                    end
                end
            end
        end
        if #self.hands[self.playerTurn] == 0
        then 
            self.playerWin = self.playerTurn
            if self.playerWin == 1 then config.score = config.score+1 end
            self.ended = true
        elseif (self:everybodyPass())
        then
            self.playerWin = "draw"
            self.ended = true
        end
    else
        -- code to manage game end
        menu = Menu:new(menuz.gameEndedMenu)
    end
end

function Game:draw()

    -- Displaying each player's hand
    if self.hands[1] then love.graphics.print("Player 1",100,0) self.hands[1]:setCoord(32,32) self.hands[1]:draw() end
    if self.hands[2] then love.graphics.print("Player 2",500,0) self.hands[2]:setCoord(432,32) self.hands[2]:draw() end
    if self.hands[3] then love.graphics.print("Player 3",100,400) self.hands[3]:setCoord(32,432) self.hands[3]:draw() end
    if self.hands[4] then love.graphics.print("Player 4",500,400) self.hands[4]:setCoord(432,432)self.hands[4]:draw() end

    -- Displaying the boneyard
    if not self.hands[3] then love.graphics.print("Boneyard",100,400) self.boneYard:setCoord(32,432) self.boneYard:draw(32,432)
    elseif not self.hands[4] then love.graphics.print("Boneyard",500,400) self.boneYard:setCoord(432,432) self.boneYard:draw(432,432)
    end

    -- Displaying the field
    love.graphics.print("Field : "..#self.field,100,248)
    self.field.ratio = (love.graphics.getWidth( ) / #self.field / 128)*0.9
    if self.field.ratio > 0.5 then self.field.ratio = 0.5 end
    self.field.x = 400 - (#self.field)*64*self.field.ratio +128
    self.field:draw()

    -- Displaying if a Player has won
    if self.playerWin
    then
        love.graphics.setColor(100,100,250)
        if self.playerWin == "draw"
        then
            love.graphics.print("Draw",256,200, 0,4,4)
        else
            love.graphics.print("Player "..self.playerWin.." wins !",256,200, 0,4,4)
        end
        love.graphics.setColor(255,255,255)
    end
end

function Game:mousepressed()

end

function Game:keypressed(key, unicode)
    if key == 'p'
    then
        game = Game:new()
    end
end

function Game:everybodyPass()
    local ret = false
    for k, v in ipairs (self.hands) do
        if not v.hasPassed
        then
            ret = false break
        else
            ret = true
        end
    end
    return ret
end
