--------------
-- Domino Deus --
--------------
-- a Domino game by Ghuntar WALDMEISTER
-- main.lua

require("BONE_class.lua")
require("BONESET_class.lua")
require("BONEYARD_class.lua")
require("FIELD_class.lua")

function love.load()

    -- Initialisation of a random seed based on os.time()
    math.randomseed(os.time())
    math.random()
    math.random()

    -- The boneyard is where players draw theire bones/tiles.
    boneYard = BoneYard:new()

    -- Generation of all the 28 bones and there insertion in the boneyard.
    for x=0 , 6 do
        for y=x, 6 do
            table.insert(boneYard,Bone:new({x,y}))
        end
    end

    -- Player's Hands
    numberOfPlayer = 3
    hands = {}
    for i=1 , numberOfPlayer do
        hands[i] = BoneSet:new()
    end

    -- Distribution of the bones
    playerTurn = 0
    while (#boneYard>0) do
        playerTurn = (playerTurn)%numberOfPlayer +1
        -- Each player receives 7 bones
        if #hands[playerTurn]>6 then break end
        key = math.random(1,#boneYard)
        table.insert(hands[playerTurn],boneYard[key])
        table.remove(boneYard,key)
        -- print("Bone "..hands[playerTurn][#hands[playerTurn]]:tostring().." from boneyard to Player"..playerTurn)
    end
    -- print("End of distribution")
    playerTurn = 0
    -- firstBone is the first bone that will be played (it should be double 6 in most case)
    if #boneYard == 0 then firstBone = Bone:new({6,6})
    elseif not Bone:new({6,6}):isPresentInTable(boneYard) then firstBone = Bone:new({6,6})
    elseif not Bone:new({5,5}):isPresentInTable(boneYard) then firstBone = Bone:new({5,5})
    elseif not Bone:new({4,4}):isPresentInTable(boneYard) then firstBone = Bone:new({4,4})
    elseif not Bone:new({3,3}):isPresentInTable(boneYard) then firstBone = Bone:new({3,3})
    elseif not Bone:new({2,2}):isPresentInTable(boneYard) then firstBone = Bone:new({2,2})
    elseif not Bone:new({1,1}):isPresentInTable(boneYard) then firstBone = Bone:new({1,1})
    elseif not Bone:new({0,0}):isPresentInTable(boneYard) then firstBone = Bone:new({0,0})
    end

    -- The field is where the bones will be played (empty at the begining).
    field = Field:new{}
    -- Looking in each player's hand the firstBone to know who begins
    for playerTurnTemp,playerHand in ipairs(hands) do
        for k,v in ipairs(playerHand) do
            if v:fullMatch(firstBone)
            then
                table.insert(field,v)
                table.remove(playerHand,k)
                -- print("Bone "..v:tostring().." from Player"..playerTurnTemp.." to field")
                playerTurn = playerTurnTemp
            end
        end
    end
    -- theses two values represent the ends of the field
    fieldFirstValue = field[1][1]
    fieldLastValue = field[1][2]

    -- A marker to know who win
    playerWin = nil
    go = false
end

function love.update(dt)
    -- if love.keyboard.isDown( " " ) then go = true end
    if go
    then
        if not playerWin
        then
            playerTurn = (playerTurn)%numberOfPlayer +1
            playerHasPlayed = false
            while not playerHasPlayed do
                for k,v in ipairs(hands[playerTurn]) do
                    if v:matchValue(fieldFirstValue)
                    then
                        table.insert(field,1,v)
                        if field[1][2] ~= field[2][1] then field[1]:switch() end
                        table.remove(hands[playerTurn],k)
                        fieldFirstValue = v:matchValue(fieldFirstValue)
                        -- print("Bone "..v:tostring().." from Player"..playerTurn.." to field")
                        playerHasPlayed = true
                        break
                    elseif v:matchValue(fieldLastValue)
                    then
                        table.insert(field,v)
                        if field[#field][1] ~= field[#field -1][2] then field[#field]:switch() end
                        table.remove(hands[playerTurn],k)
                        fieldLastValue = v:matchValue(fieldLastValue)
                        -- print("Bone ["..v[1].."]["..v[2].."] from Player"..playerTurn.." to field")
                        -- print("Bone "..v:tostring().." from Player"..playerTurn.." to field")
                        playerHasPlayed = true
                        break
                    end
                end
                if not playerHasPlayed
                then
                    if #boneYard>0
                    then
                        key = math.random(1,#boneYard)
                        table.insert(hands[playerTurn],boneYard[key])
                        table.remove(boneYard,key)
                        -- print("Bone ["..hands[playerTurn][#hands[playerTurn]][1].."]["..hands[playerTurn][#hands[playerTurn]][2].."] from boneyard to PLayer "..playerTurn)
                        playerHasPlayed = false
                    else
                        -- print("Player "..playerTurn.." can not play.")
                        playerHasPlayed = true
                    end
                end
            end
        end

        if #hands[playerTurn] == 0
        then
            playerWin = playerTurn
            -- print("Player "..playerWin.."wins !")
        end
    go = true
    end
end

function love.draw()
    if not go then
        love.graphics.print("Press Space to play !",300,256, 0,3.5,4)
    end
    -- Displaying the boneyard
    if not hands[3] then love.graphics.print("Boneyard",100,400) boneYard:draw(32,432)
    elseif not hands[4] then love.graphics.print("Boneyard",500,400) boneYard:draw(432,432)
    end

    -- Displaying each player's hand
    if hands[1] then love.graphics.print("Player 1",100,0) hands[1]:draw(32,32) end
    if hands[2] then love.graphics.print("Player 2",500,0) hands[2]:draw(432,32) end
    if hands[3] then love.graphics.print("Player 3",100,400) hands[3]:draw(32,432) end
    if hands[4] then love.graphics.print("Player 4",500,400) hands[4]:draw(432,432) end
    -- for num,hand in ipairs (hands) do
        -- love.graphics.print("Hand"..num,num*96,0)
        -- for k,v in ipairs(hand) do
            -- v:print(num*96,k*16)
            -- v:draw(num*96+16,k*22,0.3)
        -- end
        -- hand:draw(32,num*80)
    -- end

    -- Displaying the field
    -- love.graphics.print("Field",500,0)
    -- for k,v in ipairs(field) do
        -- v:print(500,k*16)
        -- v:draw(500+16,k*22,0.3)
    -- end
    field:draw(32,300)

    if playerWin
    then
        love.graphics.setColor(100,100,250)
        love.graphics.print("Player "..playerWin.." wins !",256,200, 0,4,4)
        love.graphics.setColor(255,255,255)
    end
    -- love.graphics.print("Field Ends : ["..fieldFirstValue.."]["..fieldLastValue.."]",512,512)

    -- Just display some infoz
    love.graphics.print(numberOfPlayer.." players",700,544)
    love.graphics.print(love.timer.getFPS(),700,560)
    love.graphics.print(_VERSION,700,576)
end

function love.mousepressed(x, y, button)

end

function love.keypressed(key, unicode)
    if key == ' '
    then
        go = true
    end
end
